SystemOrganization addCategory: #'Magritte-RealMemento'!

!MACachedMemento methodsFor: '*magritte-realmemento-override' stamp: 'kph 1/22/2008 18:26'!
commit
	super commit.
	model commit: [ self push: self cache ].
	self reset! !

MACachedMemento subclass: #MARealMemento
	instanceVariableNames: 'snapshot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-RealMemento'!

!MARealMemento commentStamp: 'kph 11/20/2007 06:11' prior: 0!
I cache values being read and written without touching the model. 

But unlike MACachedMemento I use a copy of the model in the cache, this enables model behaviour to work.

When committing changes, the modifications will be propagated to the model all at once.!

MARealMemento subclass: #MARealCheckedMemento
	instanceVariableNames: 'original'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-RealMemento'!

!MARealCheckedMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:19'!
hasConflict
	"Answer ==true==, if there is an edit conflict."

	^ self hasChanged and: [ (self original =@ self pullRaw) not ]! !

!MARealCheckedMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:20'!
original
	^ original! !

!MARealCheckedMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:20'!
reset
	super reset.
	self setOriginal: self pullRaw! !

!MARealCheckedMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:20'!
setOriginal: aDictionary
	original := aDictionary! !

!MARealCheckedMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:21'!
validate
	self hasConflict
		ifFalse: [ ^ super validate ].
	self reset.
	MAConflictError
		description: self description
		signal: self description conflictErrorMessage! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 10:25'!
commit
 
	cache commit: [ self push: self cache ].
	self reset! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:26'!
hasChanged
	"Answer ==true==, if the cached data is different to the data in the model."

	^ ((self pullRawFrom: self cache) =@ self pullRaw) not! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:40'!
hasChangedSinceSnapshot
	"Answer ==true==, if the cached data is different to the data in the model."

	^ ((self pullRawFrom: self cache) =@ (self snapshot))not ! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:39'!
makeSnapshot

	snapshot := self pullRawFrom: self cache! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 11:37'!
pull
	"Utitlity method to copies the model into a cache populating nil values with the defaults specified by the descriptions"

	| result |
	
	result := self model copyForRealMemento.
		
	result description do: [ :each |
		(result readUsing: each) ifNil: [ result update: each default using: each ].
  	].

	^ result ! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:25'!
pullRaw
	^ self pullRawFrom: self model! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:24'!
pullRawFrom: aModel
	| result |
	result := Dictionary new.
	self description do: [ :each |
		result at: each put: (aModel
			readUsing: each) ].
	^ result! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 1/16/2008 08:48'!
push:  acache
 
	^ model copyFrom: acache! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:15'!
readUsing: aDescription
	^ (self cache readUsing: aDescription)
		ifNil: [ aDescription default ]! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 06:41'!
snapshot

	^ snapshot! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2007 09:25'!
validate
	"Check if the data in the receiver would be valid if committed. In case of problems an exception is raised."

	self description in: [ :desc |
		"desc validator" MAValidatorVisitorForRealMemento on: self description: desc
	]! !

!MARealMemento methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 11:29'!
write: anObject using: aDescription
	self cache update: anObject using: aDescription! !

MAComponentDecoration subclass: #MACommitDecoration
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-RealMemento'!

!MACommitDecoration class methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 14:46'!
target: target

^ (self new) target: target;  yourself! !

!MACommitDecoration methodsFor: 'as yet unclassified' stamp: 'kph 11/20/2007 14:45'!
handleAnswer: anObject continueWith: aBlock
 
 	 target modelChangedTo: anObject.
	 super handleAnswer: anObject continueWith: aBlock! !

!MACommitDecoration methodsFor: 'accessing' stamp: 'kph 11/20/2007 14:44'!
target
	"Answer the value of target"

	^ target! !

!MACommitDecoration methodsFor: 'accessing' stamp: 'kph 11/20/2007 14:44'!
target: anObject
	"Set the value of target"

	target := anObject! !

!Object methodsFor: '*magritte-realmemento' stamp: 'kph 1/22/2008 18:27'!
commit: aBlock
	^ aBlock value! !

!Object methodsFor: '*magritte-realmemento' stamp: 'kph 8/21/2008 01:03'!
copyForRealMemento

	^ self shallowCopy postCopyForRealMemento ! !

!Object methodsFor: '*magritte-realmemento' stamp: 'kph 8/21/2008 01:04'!
postCopyForRealMemento ! !

!Object methodsFor: '*magritte-realmemento' stamp: 'kph 11/20/2007 11:32'!
update: anObject using: aDescription
	"Dispatch the write-access to the receiver of anObject using the accessor of aDescription."

	aDescription accessor update: anObject to: self! !

MAValidatorVisitor subclass: #MAValidatorVisitorForRealMemento
	instanceVariableNames: 'memento'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-RealMemento'!

!MAValidatorVisitorForRealMemento methodsFor: 'accessing' stamp: 'kph 12/20/2007 09:26'!
memento
	"Answer the value of memento"

	^ memento! !

!MAValidatorVisitorForRealMemento methodsFor: 'accessing' stamp: 'kph 12/20/2007 09:26'!
memento: anObject
	"Set the value of memento"

	memento := anObject! !

!MAValidatorVisitorForRealMemento methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2007 09:27'!
on: anObject description: aDescription
	memento := anObject.
	super on: anObject description: aDescription
 ! !

!MAValidatorVisitorForRealMemento methodsFor: 'as yet unclassified' stamp: 'kph 1/7/2008 09:06'!
visitDescription: aDescription
	"Validate the current object using aDescription within an exception handler to avoid running further tests that might cause error-cascades."

	(aDescription isVisibleForItem: memento) ifFalse: [ ^ self ].
	[ self validate: self object using: aDescription ]
		on: MAValidationError
		do: [ :err | err beResumable; pass ]! !

!MAStraitMemento methodsFor: '*magritte-realmemento-override' stamp: 'kph 1/25/2008 10:47'!
write: anObject using: aDescription
	self model update: anObject using: aDescription! !

!MASelectorAccessor class methodsFor: '*magritte-realmemento-override' stamp: 'kph 6/4/2008 16:33'!
selector: aSelector

	aSelector asSymbol isUnary ifFalse: [ ^ self write: aSelector ].

	^ self new
		selector: aSelector;
		yourself! !

!MAElementDescription methodsFor: '*magritte-realmemento-override' stamp: 'kph 11/20/2007 07:51'!
default
	^ (self propertyAt: #default ifAbsent: [ self class defaultDefault ]) yourself
 ! !
